#ifndef GLAREA_H
#define GLAREA_H

#include <SCIP/scip_solver.h>      // because some structs in <SCIP/scip/struct_misc.h> have a member named 'slots' which conflicts with Qt
#include <GL/glew.h>
#include <QGLWidget>
#include <QString>
#include <wrap/gui/trackball.h>
#include <wrap/qt/gl_label.h>
#include "render_data.h"

class GLArea : public QGLWidget {
    Q_OBJECT

public:
  typedef PolychordSupport<PolyMesh>                       PolychordSupport;
  typedef PolychordSupport::ScalarType                     ScalarType;
  typedef PolychordSupport::ScalarContainer                ScalarContainer;
  typedef PolychordSupport::PointType                      PointType;
  typedef PolychordSupport::PointContainer                 PointContainer;
  typedef PolychordSupport::PointConstIterator             PointConstIterator;
  typedef PolychordSupport::FlowPoints                     FlowPoints;
  typedef PolychordSupport::FlowContainer                  FlowContainer;
  typedef PolychordSupport::FlowContainerConstIterator     FlowContainerConstIterator;
  typedef PolychordSupport::FlowMap                        FlowMap;
  typedef PolychordSupport::FlowMapIterator                FlowMapIterator;
  typedef PolychordSupport::FlowMapConstIterator           FlowMapConstIterator;
  typedef PolychordSupport::PolychordPoints                PolychordPoints;
  typedef PolychordSupport::PolychordContainer             PolychordContainer;
  typedef PolychordSupport::PolychordMap                   PolychordMap;
  typedef PolychordSupport::PolychordMapIterator           PolychordMapIterator;
  typedef PolychordSupport::PolychordMapConstIterator      PolychordMapConstIterator;

  enum InputMode {
    VIEW,
    DRAW2DLINES
  };

  explicit GLArea(QWidget *parent = 0);

public slots:
    void clearButtons();
    void setRenderData(RenderData *renderData);
    void setInputMode(InputMode im);
    void setNumberOfSamples(const size_t nSamples);
    void resetTrackball();
    void setTrackballActive(bool active);
    void setTrackballVisible(bool visible);
    void takeSnapShot(const QString &filename);

protected:
    // GL functions
    void initializeGL();
    void resizeGL(int w, int h);
    void paintEvent(QPaintEvent *);
    // interaction functions
    void keyPressEvent(QKeyEvent *e);
    void keyReleaseEvent(QKeyEvent *e);
    void mousePressEvent(QMouseEvent *e);
    void mouseMoveEvent(QMouseEvent *e);
    void mouseReleaseEvent(QMouseEvent *e);
    void wheelEvent(QWheelEvent *e);

private:
    void enter2D(QPainter *painter);
    void exit2D(QPainter *painter);
    void drawRichText(QPainter *painter, const PointType &p, const QString &text, const vcg::Color4b &c);
    void prepareOpenGL(const float w, const float h);
    void pickOnPlaneGL(PointType &pickedPoint);
    n_corners_type nearestSide(const PointType &p, PointType &closestPoint, num_type &edgeIndex);
    void picking();
    void renderTriMesh(const ScalarType scaleFactor, const PointType &center);
    void renderBoundary();
    void renderData(QPainter *painter, const MeshData &meshData, const ScalarType scaleFactor, const PointType &center,
                    const ShowVertices showVertices, const bool showStrings, const bool showDrawnPolyLines);

    InputMode             _inputMode;
    vcg::Trackball        _trackball;
    bool                  _isTrackballActive;
    bool                  _showTrackball;
    RenderData           *_renderData;
    QFont                 _font;
    bool                  _hasToPick;
    bool                  _startNewPolyLine;
    bool                  _endPolyLine;
    vcg::Point2i          _picked2DPoint;
    FlowTopology          _lastFlowTopology;
    FlowPoints            _lastFlowPoints;
    size_t                _nSamples;
};

#endif // GLAREA_H
