#ifndef PATCHRETOPO_H
#define PATCHRETOPO_H

#include <SCIP/scip_solver.h>      // because some structs in <SCIP/scip/struct_misc.h> have a member named 'slots' which conflicts with Qt
#include <QScrollArea>
#include <QTextEdit>
#include <QXmlStreamReader>
#include "render_data.h"
using namespace vcg::tri::pl;

class PatchRetopo : public QScrollArea
#ifndef SINGLE_THREAD
                  , public MT_PatchPriorityLookupNotifiable
#endif
                  , public std::enable_shared_from_this<PatchRetopo> {
  Q_OBJECT

public:
  explicit PatchRetopo(RenderData *meshData,
#ifdef SINGLE_THREAD
                       PatchPriorityLookup<PolyMesh> *db,
#else
                       MT_PatchPriorityLookup<PolyMesh> *db,
#endif
                       QWidget *parent = 0);

#ifndef SINGLE_THREAD
  void notify(NotificationType notificationType,
              count_type nTemplatePatches, count_type nPatches,
              std::chrono::nanoseconds duration);

  void registerForNotifications();
#endif

  ~PatchRetopo();

signals:
  void changed();
  void closed();
  void setMin(int min);
  void setMax(int max);
  void setVal(int val);
  void patchesChangedAt(int index);
  void enable(bool en);
  void executing(bool ex);
  void toStatus(const QString &string);

public slots:
  void reset();

protected:
  void closeEvent(QCloseEvent *e);

private slots:
  void showTriMeshSlot(bool show);
  void loadXML();
  void clearInput();
  void selectSortCriterion(int c);
  void showQuality(bool show);
  void smoothWithFlowConstraints(bool smooth);
  void setMaxSingularities(int maxS);
  void setMinValence(int minV);
  void setMaxValence(int maxV);
  void setMaxConcavities(int maxC);
  void setQualityThreshold(double th);
  void setILPthreshold(int th);
  void setTolerance(int tolerance);
  void setLaplacianWeight(double weight);
  void findPatches();
  void stopSearching();
  void selectPatch(int i);

private:
  void readXML(QXmlStreamReader &xmlReader);
  void patchToMeshData(const Patch<PolyMesh> &patch, MeshData &meshData, const bool qualityColor = false);
  void printBoundaryIDs(const vcg::face::Pos<PolyFace> &startPos, const std::string &bEq, const std::vector<num_type> &bVars,
                        std::list< std::pair<var_label_type, CoordType> > &labels);

  // GUI elements
  QTextEdit                                                  *patchInfoTextEdit;
  static QString                                              logStr;
  QString                                                     description;
  QString                                                     baseDescription;
  QString                                                     msg;
  bool                                                        showQualityColor;

  // data objects
  RenderData                                                 *renderData;
#ifdef SINGLE_THREAD
  typedef PatchPriorityLookup<PolyMesh>::CompareCriteria      CompareCriteria;
  PatchPriorityLookup<PolyMesh>                              *pDBlookup;
#else
  typedef MT_PatchPriorityLookup<PolyMesh>::CompareCriteria   CompareCriteria;
  MT_PatchPriorityLookup<PolyMesh>                           *pDBlookup;
  std::chrono::nanoseconds                                    elapsedFromPreviousUIupdated;
  std::mutex                                                  mutexForTime;
#endif
  typedef PatchRetopologizer<PolyMesh, TMesh>                 PatchRetopologizer;
  typedef PatchRetopologizer::ScalarType                      ScalarType;
  typedef PatchRetopologizer::PointType                       PointType;
  typedef PatchRetopologizer::PointContainer                  PointContainer;
  typedef PatchRetopologizer::FlowMapIterator                 FlowMapIterator;
  typedef PointContainer::const_iterator                      PointContainerConstIterator;
  typedef TMesh::VertexIterator                               VertexIterator;
  typedef TMesh::FaceIterator                                 FaceIterator;
  std::shared_ptr<PatchRetopologizer>                         patchGeometrizator;
  CompareCriteria                                             compareCriteria;

  // parameters
  int                                                         currentPatch;
};

#endif // PATCHRETOPO_H
